header {
/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

    package net.christopherschultz.evaluator.parser;

    import net.christopherschultz.evaluator.*;
}

class ExpressionParserImpl extends Parser;

options {
    buildAST = false;
    k=1;
    defaultErrorHandler = false;
}

expression returns [Expression exp = null] throws EvaluationException
    :
        exp=expr (EOF!)
    ;

expr returns [Expression exp = null] throws EvaluationException
    :
        exp=atom
    ;

// Whitespace separated list of expressions
wslist returns [java.util.List items = new java.util.ArrayList()] throws EvaluationException
    {
         Expression item;
    }
    :
        LPAREN!
	((SPACE!)? item=atom { if(null != item) { items.add(item); } })*
        RPAREN!
    ;

list returns [Expression exp = null] throws EvaluationException
    {
        java.util.List items = null;
    }
    :
        items=wslist
        {
            exp = new FunctionCallExpression((Expression)items.get(0),
                                            items.subList(1, items.size()));
        }
    ;

// Symbol
symbol returns [ Expression exp = null ] throws EvaluationException
    :
        symbol:SYMBOL
        {
            exp = new IdentifierExpression(symbol.getText());
        }
    ;

/*
literal returns [Expression exp = null] throws EvaluationException
    :
        hex:HEX_LITERAL {   Object value;
                            String hexStr = hex.getText();
                            int length = hexStr.length();
                            // All these temporary objects, here, will
                            // allow us to create the correctly-sized
                            // object (byte, short, int, long) with the
                            // specified bit pattern.
//9223372036854775807

                            if(length > 10) // 8 hex + leading "0x"
                            {
                                value = Long.decode(hexStr);
                            }
                            else if(length > 6)
                            {
                                Long tl = Long.decode(hexStr);
                                if(tl.longValue() > Integer.MAX_VALUE)
                                    value = new Integer((int)(tl.longValue() - Integer.MAX_VALUE));
                                else
                                    value = new Integer(tl.intValue());
                            }
                            else if(length > 4)
                            {
                                Integer ti = Integer.decode(hexStr);
                                if(ti.intValue() > Short.MAX_VALUE)
                                    value = new Short((short)(ti.intValue() - Short.MAX_VALUE));
                                else
                                    value = new Short(ti.shortValue());
                            }
                            else
                            {
                                Short ts = Short.decode(hexStr);
                                if(ts.shortValue() > Byte.MAX_VALUE)
                                    value = new Byte((byte)(ts.shortValue() - Byte.MAX_VALUE));
                                else
                                    value = new Byte(ts.byteValue());
                            }
                            exp = new ConstantExpression(value);
                        }
        |
        oct:OCTAL_LITERAL { Object value;
                            String octalStr = oct.getText();
                            int length = octalStr.length();
                            if(length > 11) // 10 octal + leading "0"
                                value = Long.decode(octalStr);
                            else if(length > 6)
                                value = Integer.decode(octalStr);
                            else if(length > 3)
                                value = Short.decode(octalStr);
                            else
                                value = Byte.decode(octalStr);
                            exp = new ConstantExpression(value);
                        }
        |
// TODO: allow larger precision than simply 32-bit integer. Long? BigInteger?
// TODO: How can we use a BigInteger to determine if the value fits into a
// certain type of primitive? Just make BigInteger objects out of
// Integer.MAX_VALUE, etc?
        i:INT_LITERAL { exp = new ConstantExpression(new Integer(i.getText())); }
        |
        f:DECIMAL_LITERAL { exp = new ConstantExpression(new Double(f.getText())); }
        |
        s:STRING { exp = new ConstantExpression(s.getText()); }
    ;
*/

// Atom
atom returns [Expression exp = null] throws EvaluationException
    {
//        java.util.List l = null;
    }
    :
        exp=
        symbol
        |
        exp=list
        |
        hex:HEX_LITERAL {   Object value;
                            String hexStr = hex.getText();
                            int length = hexStr.length();
                            // All these temporary objects, here, will
                            // allow us to create the correctly-sized
                            // object (byte, short, int, long) with the
                            // specified bit pattern.
//9223372036854775807

                            if(length > 10) // 8 hex + leading "0x"
                            {
                                value = Long.decode(hexStr);
                            }
                            else if(length > 6)
                            {
                                Long tl = Long.decode(hexStr);
                                if(tl.longValue() > Integer.MAX_VALUE)
                                    value = new Integer((int)(tl.longValue() - Integer.MAX_VALUE));
                                else
                                    value = new Integer(tl.intValue());
                            }
                            else if(length > 4)
                            {
                                Integer ti = Integer.decode(hexStr);
                                if(ti.intValue() > Short.MAX_VALUE)
                                    value = new Short((short)(ti.intValue() - Short.MAX_VALUE));
                                else
                                    value = new Short(ti.shortValue());
                            }
                            else
                            {
                                Short ts = Short.decode(hexStr);
                                if(ts.shortValue() > Byte.MAX_VALUE)
                                    value = new Byte((byte)(ts.shortValue() - Byte.MAX_VALUE));
                                else
                                    value = new Byte(ts.byteValue());
                            }
                            exp = new ConstantExpression(value);
                        }
        |
        oct:OCTAL_LITERAL { Object value;
                            String octalStr = oct.getText();
                            int length = octalStr.length();
                            if(length > 11) // 10 octal + leading "0"
                                value = Long.decode(octalStr);
                            else if(length > 6)
                                value = Integer.decode(octalStr);
                            else if(length > 3)
                                value = Short.decode(octalStr);
                            else
                                value = Byte.decode(octalStr);
                            exp = new ConstantExpression(value);
                        }
        |
/* TODO: allow larger precision than simply 32-bit integer. Long? BigInteger? */
/* TODO: How can we use a BigInteger to determine if the value fits into a certain type of primitive? Just make BigInteger objects out of Integer.MAX_VALUE, etc? */
        i:INT_LITERAL { exp = new ConstantExpression(new Integer(i.getText())); }
        |
        f:DECIMAL_LITERAL { exp = new ConstantExpression(new Double(f.getText())); }
        |
        s:STRING { exp = new ConstantExpression(s.getText()); }
/*
        |
        // function call :)
        LPAREN! l=list RPAREN! {
            exp = new FunctionCallExpression((Expression)l.get(0),
                                             l.subList(1, l.size()));
        }
*/
    ;

class ExpressionLexerImpl extends Lexer;

options {
    k=2; // (2)needed for newline junk and relations like <, <= and >, >=
    charVocabulary = '\3'..'\377';
}

LPAREN: '('
    ;

RPAREN: ')'
    ;

STRING
    :
        DOUBLEQUOTE!
        (CHAR_ESC | ~('\"' | '\\') )*
        DOUBLEQUOTE!
    ;

protected
SYMBOL_START
    :
        (~('('|')'|'0'..'9'|'.'|'"'|' '|'\r'|'\n'|'\t'))
    ;

protected
SYMBOL_REST
    :
    SYMBOL_START
    |
    DIGIT
    |
    DOT
    ;

SYMBOL
    :
        SYMBOL_START
        (SYMBOL_REST)*
    ;

NUMERIC_LITERAL:
        (ZERO (OCTAL_DIGIT)+) => OCTAL_LITERAL { $setType(OCTAL_LITERAL); }
        |
        (ZERO Xx) => HEX_LITERAL  { $setType(HEX_LITERAL); }
        |
        ( DOT DIGITS) => DECIMAL_LITERAL { $setType(DECIMAL_LITERAL); }
        |
        ( DIGITS DOT (DIGITS)? ) => DECIMAL_LITERAL { $setType(DECIMAL_LITERAL); }
        |
        ( DIGITS)    { $setType(INT_LITERAL); }
        ;

SPACE: ( WS )+ ;

protected CHAR_ESC
    :
        '\\'
        ( 'n'  { $setText("\n"); }
        | 'r'  { $setText("\r"); }
        | 't'  { $setText("\t"); }
        | 'b'  { $setText("\b"); }
        | 'f'  { $setText("\f"); }
        | '\"' { $setText("\""); }
        | '\'' { $setText("\'"); }
        | '\\' { $setText("\\"); }
        )
    ;


protected PLUS: '+' ;

protected MINUS: '-';

protected SIGN: ( PLUS | MINUS ) ;

protected OCTAL_LITERAL: (ZERO (OCTAL_DIGIT)+) ;

protected OCTAL_DIGIT: '0'..'7' ;

protected HEX_LITERAL :  (ZERO Xx (HEX_DIGIT)+) ;

protected ZERO: '0' ;

protected Xx: ('x' | 'X') ;

protected HEX_DIGIT: ( 'a'..'z' | 'A'..'Z' | DIGIT ) ;

protected INT_LITERAL: DIGITS ;

protected DECIMAL_LITERAL: ( ( DIGITS )? DOT ( DIGITS )? ) ;

protected DIGITS : ( DIGIT )+ ;

protected DIGIT: '0'..'9' ;

protected DOT: '.' ;

protected
DOUBLEQUOTE: '\"'
    ;

protected
WS:
        (   ' '
        |   '\t'
        |   '\r' '\n' { newline(); }
        |   '\r'      { newline(); }
        |   '\n'      { newline(); }
        )
        {$setType(Token.SKIP);} //ignore this token
    ;
