/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

/**
 * An Expression which represents an identifier.
 *
 * @author Chris Schultz
 * @version $Revision: 1.5 $ $Date: 2008-06-20 22:58:48 $
 */
public class IdentifierExpression
    extends ConstantExpression // TODO: Is this cheating?
{
    /**
     * Creates a new IdentifierExpression with the given identifier.
     *
     * @param identifier The identifier for this IdentifierExpressions.
     *
     * @throws IllegalArgumentException If the identifier is <code>null</code>
     *                                  or empty.
     */
    public IdentifierExpression(String identifier)
    {
        super(identifier);

        if(null == identifier || 0 == identifier.trim().length())
            throw new IllegalArgumentException("Identifier must have a value");
    }

    /**
     * Gets the symbol used by this identifier.
     */
    public String getSymbol()
    {
        return (String)super.getValue();
    }

    /**
     * Gets the symbol used by this identifier.
     */
    public String getIdentifier()
    {
        return (String)super.getValue();
    }

    public void acceptVisitor(ExpressionVisitor visitor)
    {
        visitor.visitIdentifierExpression(this);
    }

    public Object evaluate(EvaluationContext ec)
        throws EvaluationException
    {
        return ec.get((String)super.getValue());
    }

    public String toString()
    {
        return "<" + this.getSymbol() + ">";
    }
}
