/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.ArrayList;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Executes a function on the remaining parameters, returning an array.
 * </p>
 *
 * <p>
 *   Usage: distribute(f: Function, arg1, ..., argN)
 * </p>
 *
 * <p>
 *    Returns an array containing all of the results of calling the function
 *    <code>f</code> for each other argument individually.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class DistributeFunction
    extends StandardFunction
{
    public Object call(EvaluationContext ec,
		       String functionName,
		       Object[] arguments)
	throws EvaluationException
    {
	if(null == arguments || 2 > arguments.length)
	    throw new EvaluationException("Expected at least 2 arguments, got "
					  + (null == arguments ? "0" : String.valueOf(arguments.length)));

	Function func = (Function)arguments[0];

	Object[] results = new Object[arguments.length - 1];
	ArrayList arg = new ArrayList(1);
	arg.add(null); // Prime the array

	for(int i=0; i<results.length; ++i)
	{
	    arg.set(0, arguments[i+1]);

	    results[i] = func.call(ec,
				   null, // function name?
				   arg);
	}

	return results;
    }
}
/*
set("dist", create("net.christopherschultz.evaluator.function.DistributeFunction"))
dist(isNull, null, true, a, "s", 1, 2, 3, 4)
*/
