/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Returns a list containing the arguments to the function.
 * </p>
 *
 * <p>
 *   Usage: list(arg1, arg2, ...)
 * </p>
 *
 * <p>
 *    Returns a list containing all of the function's arguments.
 *    An empty argument list returns an empty list.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.1 $ $Date: 2009-04-09 15:00:39 $
 */
public class ListFunction
    implements Function
{
    public Object call(EvaluationContext ec,
                       String functionName,
                       List argumentExpressions)
        throws EvaluationException
    {
        if(null == argumentExpressions)
            return new ArrayList();

        ArrayList values = new ArrayList(argumentExpressions.size());

        for(Iterator i=argumentExpressions.iterator(); i.hasNext(); )
            values.add(StandardFunction.getExpressionValue(ec,
                                                           i.next()));

        return values;
    }
}
/*
(set quote (create "net.christopherschultz.evaluator.function.QuoteFunction"))
(set list (create "net.christopherschultz.evaluator.function.ListFunction"))


 */
