/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.HashMap;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Returns the current memory usage as a Map of descriptions to Integers.
 * </p>
 *
 * <p>
 *    This method takes no arguments.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.1 $ $Date: 2005/07/16 17:25:21 $
 */
public class MemoryFunction
    extends StandardFunction
{
    public Object call(EvaluationContext ec,
		       String functionName,
		       Object[] arguments)
	throws EvaluationException
    {
	Runtime rt = Runtime.getRuntime();

	long total = rt.totalMemory();
	long max   = rt.maxMemory();
	long free  = rt.freeMemory();
	    
	HashMap info = new HashMap();

	info.put("free", new Long(free));
	info.put("max", new Long(max));
	info.put("total", new Long(total));
		 
	return info;
    }
}
