/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.List;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Returns the symbol as a ConstantExpression instead of evaluating it.
 * </p>
 *
 * <p>
 *   Usage: quote(identifier)
 * </p>
 *
 * <p>
 *    Returns the identifier itself.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2007-05-05 21:39:24 $
 */
public class QuoteFunction
    implements Function
{
    public Object call(EvaluationContext ec,
                       String functionName,
                       List argumentExpressions)
        throws EvaluationException
    {
        if(null == argumentExpressions || 1 != argumentExpressions.size())
            throw new EvaluationException("Expected 1 argument, got "
                                          + (null == argumentExpressions ? "0" : String.valueOf(argumentExpressions.size())));

        return (Expression)argumentExpressions.get(0);
    }
}
