/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.List;
import java.util.Iterator;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.IdentifierExpression;

/**
 * <p>
 *    Removes the value for any number of identifiers.
 *    Note that this is different than setting their values to
 *    <code>null</code>.
 * </p>
 *
 * <p>
 *   Usage: unset(id [id id id...])
 * </p>
 *
 * <p>
 *    Returns the old value for the last identifier mentioned or
 *    <code>null</code> if that identifier has no value.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class UnsetVariableFunction
    implements Function
{
    public Object call(EvaluationContext ec,
                       String functionName,
                       List argumentExpressions)
        throws EvaluationException
    {
        Object value = null;

        if(null != argumentExpressions && 0 < argumentExpressions.size())
            for(Iterator i=argumentExpressions.iterator(); i.hasNext(); )
            {
                Expression varNameExp = (Expression)i.next();

                // Can only remove the values of identifiers / symbols
                if(!(varNameExp instanceof IdentifierExpression))
                    throw new EvaluationException("Cannot remove a non-identifier: " + varNameExp);

                String varName = (String)((IdentifierExpression)varNameExp).getValue();

                value = ec.remove(varName);
            }

        return value;
    }
}
