/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.tools;

import java.io.*;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.parser.ExpressionParser;
import net.christopherschultz.evaluator.parser.ParseException;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * A command-line driver for demonstrating the expression evaluator.
 *
 * @author Chris Schultz
 * @version $Revision: 1.6 $ $Date: 2008-06-20 22:58:48 $
 */
public class Main
{
    public static void main(String[] args)
	throws IOException
    {
	System.out.println("Enter expressions, one per line, to evaluate");

	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	String line;
	ExpressionParser parser = new ExpressionParser();

	EvaluationContext ctx = new DefaultEvaluationContext().loadStandard();

	while(null != (line = in.readLine()))
	{
	    if(0 == line.trim().length())
		continue;
	    try
	    {
		Expression exp = parser.parse(line);

		if(null == exp)
		    continue;

		System.out.println("Parsed: " + exp);

		Object result = exp.evaluate(ctx);

		System.out.print("Result: " + getValue(result));
		if(null != result)
		    System.out.println(" (" + result.getClass().getName() + ")");
		else
		    System.out.println();
	    }
	    catch(ParseException pe)
	    {
		pe.printStackTrace();
	    }
	    catch(EvaluationException ee)
	    {
		ee.printStackTrace();
	    }
	    catch(RuntimeException e)
	    {
		e.printStackTrace();
	    }
	}
    }

    private static String getValue(Object o)
    {
	if(null == o)
	    return null;

	if(o.getClass().isArray())
	{
	    Class componentType = o.getClass().getComponentType();

	    if(componentType.isPrimitive())
	    {
		return o.toString(); // TODO: ?
	    }
	    else
		return java.util.Arrays.asList((Object[])o).toString();
	}
	else
	    return o.toString();
    }
}
