/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.Collections;

import net.christopherschultz.evaluator.ConstantExpression;
import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.FunctionCallExpression;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * Unit test for the isNull function.
 *
 * @author Chris Schultz
 * @version $Revision: 1.5 $ $Date: 2008-06-20 22:58:48 $
 */
public class IsNullTests
    extends TestCase
{
    Expression exp;
    EvaluationContext ec;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("isNull", new IsNullFunction());
        ec.set("null", null);
    }

    public void testNull()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new IdentifierExpression("null")));

        Assert.assertEquals(Boolean.TRUE,
                            exp.evaluate(ec));

        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(null));

        Assert.assertEquals(Boolean.TRUE,
                            exp.evaluate(ec));

        //evaluate("isNull(null)", ec));
    }

    public void testIndirectNull()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new IdentifierExpression("o")));

        // Note that object "o" has no value
        Assert.assertEquals(Boolean.TRUE,
                            exp.evaluate(ec));
    }

    public void testIndirectNonNull()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new IdentifierExpression("o")));

        // Note that object "o" has a value:
        ec.set("o", Boolean.FALSE);

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    public void testBoolean()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new ConstantExpression(Boolean.TRUE)));

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    public void testDouble()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new ConstantExpression(new Double(1.0))));

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    public void testInteger()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new ConstantExpression(new Integer(1))));

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    public void testString()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new ConstantExpression("string")));

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    public void testEmptyString()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("isNull"),
                                         Collections.singletonList(new ConstantExpression("")));

        Assert.assertEquals(Boolean.FALSE,
                            exp.evaluate(ec));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public IsNullTests(String name)
    {
        super(name);
    }
}
