/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.jmock.MockObjectTestCase;
import org.jmock.Mock;

import java.util.ArrayList;
import java.util.List;

import net.christopherschultz.evaluator.*;
import net.christopherschultz.evaluator.parser.ExpressionParser;
import net.christopherschultz.evaluator.parser.ParseException;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * UnitTest for ListFunction.
 *
 * @author Chris Schultz
 * @version $Revision: 1.1 $ $Date: 2009-04-09 15:00:39 $
 */
public class ListFunctionTests
    extends net.christopherschultz.evaluator.UnitTest
{
    EvaluationContext ec;
    Object result;
	ExpressionParser parser = new ExpressionParser();

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("list", new ListFunction());
    }

    protected Object evaluate(String expression)
        throws EvaluationException
    {
        try
        {
            Expression exp = parser.parse(expression);

            return exp.evaluate(ec);
        }
        catch (ParseException pe)
        {
            throw new EvaluationException("Cannot parse expression", pe);
        }
    }

    public void testSimpleList()
        throws EvaluationException
    {
        result = evaluate("list(1,2,3)");

        Assert.assertTrue("ListFunction does nto return a List",
                          result instanceof List);

        Assert.assertEquals("List function returns incorrectly sized list",
                            3, ((List)result).size());
        Assert.assertEquals("List function does not work",
                            new Integer(1), ((List)result).get(0));
        Assert.assertEquals("List function does not work",
                            new Integer(2), ((List)result).get(1));
        Assert.assertEquals("List function does not work",
                            new Integer(3), ((List)result).get(2));
    }

    public void testEmptyList()
        throws EvaluationException
    {
        result = evaluate("list()");

        Assert.assertTrue("ListFunction does nto return a List",
                          result instanceof List);

        Assert.assertEquals("List function returns incorrectly sized list",
                            0, ((List)result).size());
    }

    public void testNullInList()
        throws EvaluationException
    {
        result = evaluate("list(null)");

        Assert.assertTrue("ListFunction does nto return a List",
                          result instanceof List);

        Assert.assertEquals("List function returns incorrectly sized list",
                            1, ((List)result).size());

        Assert.assertNull("List function does not work",
                          ((List)result).get(0));
    }

    public void testMixedList()
        throws EvaluationException
    {
        result = evaluate("list(1,2.0,\"3\")");

        Assert.assertTrue("ListFunction does nto return a List",
                          result instanceof List);

        Assert.assertEquals("List function returns incorrectly sized list",
                            3, ((List)result).size());
        Assert.assertEquals("List function does not work",
                            new Integer(1), ((List)result).get(0));
        Assert.assertEquals("List function does not work",
                            new Double(2.0), ((List)result).get(1));
        Assert.assertEquals("List function does not work",
                            "3", ((List)result).get(2));
    }

    // Just a toy
    public void testListContains()
        throws EvaluationException
    {
        ec.set("containsAny", new net.christopherschultz.evaluator.function.ContainsAnyFunction());

        result = evaluate("containsAny(list(1,2.0,\"3\"), 3,2,1)");

        Assert.assertEquals("", Boolean.TRUE, result);
    }

    public void testListContains2()
        throws EvaluationException
    {
        ec.set("containsAny", new net.christopherschultz.evaluator.function.ContainsAnyFunction());

        result = evaluate("containsAny(list(3,2,1), 1,2.0,\"3\")");

        Assert.assertEquals("", Boolean.TRUE, result);
    }

    /**
     * Returns a Test which will execute all the test methods found
     * in this class.<p>
     *
     * @return A Test for JUnit to run.
     */
    public static Test suite()
    {
        TestSuite ts = new TestSuite(ListFunctionTests.class);

        return ts;
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public ListFunctionTests(String name)
    {
        super(name);
    }
}
