/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Arrays;
import java.util.Collections;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import net.christopherschultz.evaluator.ConstantExpression;
import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.FunctionCallExpression;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * Unit test for the set function.
 *
 * @author Chris Schultz
 * @version $Revision: 1.4 $ $Date: 2007-05-08 23:09:20 $
 */
public class SetVariableFunctionTests
    extends TestCase
{
    Expression exp;
    EvaluationContext ec;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("set", new SetVariableFunction());
        ec.set("quote", new QuoteFunction());
    }

    public void testSetIndentifier()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("set"),
                                         Arrays.asList(new Expression[] {
                                             new IdentifierExpression("foo"),
                                             new ConstantExpression(new Integer(12))
                                         }));

        exp.evaluate(ec);

        Assert.assertEquals("SetVariable does not set",
                            new Integer(12), ec.get("foo"));
    }

    public void testSetIndirectIdentifier()
        throws Exception
    {
        exp = new FunctionCallExpression(new IdentifierExpression("set"),
                                         Arrays.asList(new Expression[] {
                                             new FunctionCallExpression(new IdentifierExpression("quote"), Collections.singletonList(new IdentifierExpression("foo"))),
                                             new ConstantExpression(new Integer(12))
                                         }));

        exp.evaluate(ec);

        Assert.assertEquals("SetVariable does not set indirect identifiers",
                            new Integer(12), ec.get("foo"));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public SetVariableFunctionTests(String name)
    {
        super(name);
    }
}
